/*
 * HTMLTranslationFormatter.java
 *
 * Created on March 5, 2007, 2:06 PM
 *
 */

package info.heinzelnisse.he;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

/**
 *
 * @author Heiko Klein
 */
public class HTMLTranslationFormatter {
    static private Logger logger = Logger.getLogger(HTMLTranslationFormatter.class.getName());
    
    /** Creates a new instance of HTMLTranslationFormatter */
    public HTMLTranslationFormatter() {
    }

    /**
     *
     * @param sb StringBuilder to append messages to
     * @param tg translationGroup to be converted
     */
    public void format(StringBuilder sb, TranslationGroup tg) {
        long size = tg.size();
        if (size == 0) {
            // TODO: use localization
            sb.append(java.util.ResourceBundle.getBundle("info/heinzelnisse/he/properties").getString("word_not_found"));
        } else if (size > HeinzelTable.MAX_DISPLAYABLE_ITEMS) {
            String msg = java.util.ResourceBundle.getBundle("info/heinzelnisse/he/properties").getString("found_to_many");
            MessageFormat formatter = new MessageFormat("");
            formatter.applyPattern(msg);
            sb.append(formatter.format(new Object[]{new Long(size)}));
        } else {
            if (tg.getGermanTranslation() != null) {
                formatTranslationCollection(sb, tg.getGermanTranslation());
            }
            if (tg.getNorwegianTranslation() != null) {
                if (tg.getGermanTranslation() != null) {
                    sb.append("<hr></hr>");
                }
                formatTranslationCollection(sb, tg.getNorwegianTranslation());
            }
            if (tg.getMixedTranslation() != null) {
                formatTranslationCollection(sb, tg.getMixedTranslation());
            }
        }
        logger.fine("formattedMessage:" + sb);
    }
    
    private void formatWordCollection(StringBuilder sb, Collection<Translation> wordList, String color) {
        if (wordList.size() == 0) return;
        sb.append("<p style=\"background-color: " + color + "; margin-top: 0pt; margin-bottom: 2pt\">");
        int i = 0;
        for (Iterator<Translation> it = wordList.iterator(); it.hasNext(); i++) {
            Translation trans = it.next();
            if (i == 0) { // add the word to translate only once
                sb.append("<b>");
                sb.append(trans.getWord());
                if (!"".equals(trans.getArticle()))
                    sb.append(" <em>(" + trans.getArticle() + ")</em>");
                if (!"".equals(trans.getOther()))
                    sb.append(" <em>"+trans.getOther() + "</em>");
                sb.append("</b> ");
            }
            // add now all translations
            sb.append(trans.getTransWord());
            if (!"".equals(trans.getTransArticle()))
                sb.append(" <em>(" + trans.getTransArticle() + ")</em>");
            if (!"".equals(trans.getTransOther()))
                sb.append(" <em>" + trans.getTransOther() + "</em>");
            
            if (it.hasNext()) {
                sb.append("; ");
            }
        }
        sb.append("</p>");
    }

    /**
     *
     * @param sb
     * @param tCollection
     * @return index of first found exact translation in stringbuilder, might be null
     *    if no exact has been found
     */
    private void formatTranslationCollection(StringBuilder sb, Collection<Translation> tCollection) {
        List<Translation> tList = new ArrayList<Translation>(tCollection);
        logger.info("got "+tList.size() + " trans");
        Collections.sort(tList);
        String nuId = tList.get(0).getNonUniqueIdentifier();
        List<Translation> wordList = new ArrayList<Translation>();
        Integer firstExact = null;
        final String white = "#ffffff";
        final String gray1 = "#dddddd";
        final String gray2 = "#eeeeee";
        String color = white;
        for (Translation t : tList) {
            if (nuId.equals(t.getNonUniqueIdentifier())) {
                wordList.add(t);
            } else {
                formatWordCollection(sb, wordList, color);
                wordList.clear();
                // get a new color
                if (t.isIsExact()) {
                    color = white;
                } else {
                    // alternate gray1 and gray2, starting with gray1
                    if (color.equals(gray1)) {
                        color = gray2;
                    } else {
                        color = gray1;
                    }
                }
                nuId = t.getNonUniqueIdentifier();
                wordList.add(t);
            }
        }
        formatWordCollection(sb, wordList, color);
    }
}
