/*
 * HeinzelJFrame.java
 *
 * Created on January 20, 2007, 1:44 PM
 */
package info.heinzelnisse.he;

import java.awt.AWTException;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.MenuItem;
import java.awt.Point;
import java.awt.PopupMenu;
import java.awt.SystemTray;
import java.awt.Toolkit;
import java.awt.TrayIcon;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.jnlp.BasicService;
import javax.jnlp.FileContents;
import javax.jnlp.PersistenceService;
import javax.jnlp.ServiceManager;
import javax.jnlp.ClipboardService;
import javax.jnlp.UnavailableServiceException;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

/**
 *
 * @author  Heiko Klein
 */
public class HeinzelJFrame extends javax.swing.JFrame {

    static final String DIMENSION_PREFERENCES = "HeinzelJFrame_dimensionLocation";

    /** Creates new form HeinzelJFrame */
    public HeinzelJFrame() {
        initComponents();
        initDynamicComponents();
        setWebStartPreferences();
    }

    /**
     * Init dynamic components, i.e. look and feel installation dependent
     * parts
     * 
     */
    private void initDynamicComponents() {
        UIManager.LookAndFeelInfo[] lnfs = UIManager.getInstalledLookAndFeels();
        jRadioButtonMenuItems = new javax.swing.JRadioButtonMenuItem[lnfs.length];
        int i = 0;
        for (UIManager.LookAndFeelInfo lfi : lnfs) {
            javax.swing.JRadioButtonMenuItem current = new javax.swing.JRadioButtonMenuItem();
            jRadioButtonMenuItems[i++] = current;
            buttonGroup1.add(current);
            current.setSelected(false);
            current.setText(lfi.getName());
            current.setActionCommand(lfi.getName());
            current.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jRadioButtonLookNFeelActionPerformed(evt);
                }
            });
            jMenu5.add(current);
        }
    }
    private void setLookAndFeel(String str) {
        UIManager.LookAndFeelInfo[] lnfs = UIManager.getInstalledLookAndFeels();
        for (UIManager.LookAndFeelInfo lfi : lnfs) {
            if (lfi.getName().equals(str)) {
                try {
                    UIManager.setLookAndFeel(lfi.getClassName());
                    SwingUtilities.updateComponentTreeUI(this);
                    pack();
                    return;
                } catch (Exception ex) {
                    Logger.getLogger(HeinzelJFrame.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
        Logger.getLogger(HeinzelJFrame.class.getName()).log(Level.WARNING, "cannot set look and feel: " + str);
    }
    private void jRadioButtonLookNFeelActionPerformed(java.awt.event.ActionEvent evt) {
        setLookAndFeel(evt.getActionCommand());
    }
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonGroup1 = new javax.swing.ButtonGroup();
        heinzelJPanel1 = new info.heinzelnisse.he.HeinzelJPanel();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        jMenuItem1 = new javax.swing.JMenuItem();
        jMenu3 = new javax.swing.JMenu();
        cutNPasteCheckBoxMenuItem2 = new javax.swing.JCheckBoxMenuItem();
        jMenuItem4 = new javax.swing.JMenuItem();
        jMenu4 = new javax.swing.JMenu();
        jCheckBoxMenuItem1 = new javax.swing.JCheckBoxMenuItem();
        jMenu5 = new javax.swing.JMenu();
        jMenu2 = new javax.swing.JMenu();
        jMenuItem2 = new javax.swing.JMenuItem();
        jMenuItem3 = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Heinzelnisse");
        setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        setIconImage(Toolkit.getDefaultToolkit().getImage(HeinzelJFrame.class.getResource("resources/heinzelIcon.png")));
        addWindowFocusListener(new java.awt.event.WindowFocusListener() {
            public void windowGainedFocus(java.awt.event.WindowEvent evt) {
                formWindowGainedFocus(evt);
            }
            public void windowLostFocus(java.awt.event.WindowEvent evt) {
            }
        });
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        jMenu1.setMnemonic(java.util.ResourceBundle.getBundle("info/heinzelnisse/he/properties").getString("File").charAt(0));
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("info/heinzelnisse/he/properties"); // NOI18N
        jMenu1.setText(bundle.getString("File")); // NOI18N

        jMenuItem1.setMnemonic(java.util.ResourceBundle.getBundle("info/heinzelnisse/he/properties").getString("Quit").charAt(0));
        jMenuItem1.setText(bundle.getString("Quit")); // NOI18N
        jMenuItem1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem1ActionPerformed(evt);
            }
        });
        jMenu1.add(jMenuItem1);

        jMenuBar1.add(jMenu1);

        jMenu3.setMnemonic(java.util.ResourceBundle.getBundle("info/heinzelnisse/he/properties").getString("Edit").charAt(0));
        jMenu3.setText(bundle.getString("Edit")); // NOI18N

        cutNPasteCheckBoxMenuItem2.setText(bundle.getString("CutnPaste")); // NOI18N
        cutNPasteCheckBoxMenuItem2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cutNPasteCheckBoxMenuItem2ActionPerformed(evt);
            }
        });
        jMenu3.add(cutNPasteCheckBoxMenuItem2);

        jMenuItem4.setMnemonic(java.util.ResourceBundle.getBundle("info/heinzelnisse/he/properties").getString("Search").charAt(0));
        jMenuItem4.setText(bundle.getString("Search")); // NOI18N
        jMenuItem4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem4ActionPerformed(evt);
            }
        });
        jMenu3.add(jMenuItem4);

        jMenuBar1.add(jMenu3);

        jMenu4.setText(bundle.getString("View")); // NOI18N

        jCheckBoxMenuItem1.setText("Use SystemTray");
        jCheckBoxMenuItem1.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                jCheckBoxMenuItem1ItemStateChanged(evt);
            }
        });
        jMenu4.add(jCheckBoxMenuItem1);

        jMenu5.setText("Look 'n Feel"); // NOI18N
        jMenu4.add(jMenu5);

        jMenuBar1.add(jMenu4);

        jMenu2.setMnemonic(java.util.ResourceBundle.getBundle("info/heinzelnisse/he/properties").getString("Help").charAt(0));
        jMenu2.setText(bundle.getString("Help")); // NOI18N

        jMenuItem2.setMnemonic(java.util.ResourceBundle.getBundle("info/heinzelnisse/he/properties").getString("Usage").charAt(0));
        jMenuItem2.setText(bundle.getString("Usage")); // NOI18N
        jMenuItem2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem2ActionPerformed(evt);
            }
        });
        jMenu2.add(jMenuItem2);

        jMenuItem3.setMnemonic(java.util.ResourceBundle.getBundle("info/heinzelnisse/he/properties").getString("About").charAt(0));
        jMenuItem3.setText(bundle.getString("About")); // NOI18N
        jMenuItem3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem3ActionPerformed(evt);
            }
        });
        jMenu2.add(jMenuItem3);

        jMenuBar1.add(jMenu2);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(heinzelJPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 327, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(heinzelJPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 118, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        Point p = getLocation();
        Dimension d = getSize();
        try {
            PersistenceService service = (PersistenceService) ServiceManager.lookup("javax.jnlp.PersistenceService");
            BasicService bs = (BasicService) ServiceManager.lookup("javax.jnlp.BasicService");
            URL url = new URL(bs.getCodeBase() + DIMENSION_PREFERENCES);
            FileContents fc = null;
            try {
                fc = service.get(url);
            } catch (FileNotFoundException fnfe) {
                service.create(url, 256);
                fc = service.get(url);
            }
            BufferedWriter ow = new BufferedWriter(new OutputStreamWriter(fc.getOutputStream(true)));
            ow.append("WindowCoordinates: " + (int) d.getWidth() + "x" + (int) d.getHeight() + "+" + p.x + "+" + p.y);
            ow.newLine();
            ow.append("DividerLocation: " + heinzelJPanel1.getDividerLocation());
            ow.newLine();
            ow.append("LookAndFeel: " +UIManager.getLookAndFeel().getName());
            ow.newLine();
            if (trayIcon != null) {
                ow.append("SystemTray: true");
                ow.newLine();
            }
            if (cutNPasteCheckBoxMenuItem2.isSelected()) {
                ow.append("CutNPaste: true");
                ow.newLine();
            }
            ow.close();
        } catch (UnavailableServiceException unavEx) {
            // no webstart ...
        } catch (Exception e) {
            e.printStackTrace();
        }
    }//GEN-LAST:event_formWindowClosing

    private void setWebStartPreferences() {
        try {
            PersistenceService service = (PersistenceService) ServiceManager.lookup("javax.jnlp.PersistenceService");
            BasicService bs = (BasicService) ServiceManager.lookup("javax.jnlp.BasicService");
            URL url = new URL(bs.getCodeBase() + DIMENSION_PREFERENCES);
            FileContents fc = service.get(url);
            BufferedReader br = new BufferedReader(new InputStreamReader(fc.getInputStream()));
            String line;
            Pattern windowPattern = Pattern.compile("WindowCoordinates:\\s+(\\d+)x(\\d+)\\+(\\d+)\\+(\\d+)");
            Pattern dividerPattern = Pattern.compile("DividerLocation:\\s+(\\d+)");
            Pattern lnfPattern = Pattern.compile("LookAndFeel:\\s+(\\S+)");
            Pattern sysTrayPattern = Pattern.compile("SystemTray:\\s+(\\S+)");
            Pattern cutNPastePattern = Pattern.compile("CutNPaste:\\s+(\\S+)");
            while ((line = br.readLine()) != null) {
                Matcher windowMatcher = windowPattern.matcher(line);
                Matcher dividerMatcher = dividerPattern.matcher(line);
                Matcher lnfMatcher = lnfPattern.matcher(line);
                Matcher sysTrayMatcher = sysTrayPattern.matcher(line);
                Matcher cutNPasteMatcher = cutNPastePattern.matcher(line);
                if (windowMatcher.find()) {
                    int width = Integer.parseInt(windowMatcher.group(1));
                    int height = Integer.parseInt(windowMatcher.group(2));
                    int x = Integer.parseInt(windowMatcher.group(3));
                    int y = Integer.parseInt(windowMatcher.group(4));
                    this.setLocation(new Point(x, y));
                    this.setSize(new Dimension(width, height));
                    this.setPreferredSize(new Dimension(width, height));
                    Logger.getLogger(HeinzelJFrame.class.getName()).fine("WindowCoordinates: " + width + "x" + height + "+" + x + "+" + y);
                } else if (dividerMatcher.find()) {
                    heinzelJPanel1.setDividerLocation(Integer.parseInt(dividerMatcher.group(1)));
                    Logger.getLogger(HeinzelJFrame.class.getName()).fine("DividerLocation: " + dividerMatcher.group(1));
                } else if (lnfMatcher.find()) {
                    setLookAndFeel(lnfMatcher.group(1));
                    Logger.getLogger(HeinzelJFrame.class.getName()).fine("LookAndFeel: " + lnfMatcher.group(1));
                } else if (sysTrayMatcher.find()) {
                    Logger.getLogger(HeinzelJFrame.class.getName()).fine("SystemTray: " + sysTrayMatcher.group(1));
                    if (sysTrayMatcher.group(1).equals("true")) {
                        jCheckBoxMenuItem1.setSelected(true);
                    }
                } else if (cutNPasteMatcher.find()) {
                    Logger.getLogger(HeinzelJFrame.class.getName()).fine("CutNPaste: " + cutNPasteMatcher.group(1));
                    if (cutNPasteMatcher.group(1).equals("true")) {
                        cutNPasteCheckBoxMenuItem2.setSelected(true);
                        ControllerBean.getInstance().setCutnpaste(true);
                    } // default is off
                } else {
                    Logger.getLogger(HeinzelJFrame.class.getName()).warning("unknown preference: "+line);
                }
            }
            br.close();
        } catch (UnavailableServiceException unavEx) {
            Logger.getLogger(HeinzelJFrame.class.getName()).info("PersistenceService: "+unavEx.toString());
        } catch (FileNotFoundException fnfe) {
            // first write ?
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void jMenuItem4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem4ActionPerformed
        heinzelJPanel1.openSearchWindow();
    }//GEN-LAST:event_jMenuItem4ActionPerformed

    private void showHelpWindow(String type) {
        try {
            String filename = java.util.ResourceBundle.getBundle("info/heinzelnisse/he/properties").getString(type + ".filename");
            URL url = HeinzelJFrame.class.getResource("resources/" + filename);
            HelpWindow hw = new HelpWindow();
            hw.setHelpURL(url);
            hw.setVisible(true);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }
    private void jMenuItem3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem3ActionPerformed
        showHelpWindow("about");

    }//GEN-LAST:event_jMenuItem3ActionPerformed

    private void jMenuItem2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem2ActionPerformed
        showHelpWindow("usage");
    }//GEN-LAST:event_jMenuItem2ActionPerformed

    private void jMenuItem1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem1ActionPerformed
        System.exit(0);
    }//GEN-LAST:event_jMenuItem1ActionPerformed

    private void formWindowGainedFocus(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowGainedFocus
        // try to read the search-string from the clipboard
        if (!ControllerBean.getInstance().isCutnpaste()) {
            return;
        }
        String clipStr = null;
        Transferable contents = null;
        ClipboardService cs = getClipboardService();
        if (cs != null) {
            contents = cs.getContents();
        } else {
            // Try to use the standard clipboard if jnlp not available
            Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
            contents = clipboard.getContents(null);
        }
        boolean hasTransferableText = (contents != null) && contents.isDataFlavorSupported(DataFlavor.stringFlavor);
        if (hasTransferableText) {
            try {
                clipStr = (String) contents.getTransferData(DataFlavor.stringFlavor);
            } catch (UnsupportedFlavorException ex) {
                Logger.getLogger(HeinzelJPanel.class.getName()).log(Level.SEVERE, null, ex);
            } catch (IOException ex) {
                Logger.getLogger(HeinzelJPanel.class.getName()).log(Level.SEVERE, null, ex);
            }
            if (clipStr != null) {
                clipStr.trim();
                if (clipStr.length() > 0) {
                    ControllerBean.getInstance().setSearchString(clipStr);
                }
            }
        }
    }//GEN-LAST:event_formWindowGainedFocus

    private void jCheckBoxMenuItem1ItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_jCheckBoxMenuItem1ItemStateChanged
        if (isWebstart()) {
            // SystemTray not working with webstart
            jCheckBoxMenuItem1.setSelected(false);
            jCheckBoxMenuItem1.setEnabled(false);            
        }
        if (jCheckBoxMenuItem1.isSelected()) {
            if (SystemTray.isSupported()) {
                // get the SystemTray instance
                SystemTray tray = SystemTray.getSystemTray();
                // load an image
                Image image = Toolkit.getDefaultToolkit().getImage(HeinzelJFrame.class.getResource("resources/heinzelIcon.png"));
                // create a action listener to listen for default action executed on the tray icon
                ActionListener listener = new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        setVisible(true);
                    }
                };
                // create a popup menu
                PopupMenu popup = new PopupMenu();
                // create menu item for the default action
                MenuItem defaultItem = new MenuItem();
                defaultItem.addActionListener(listener);
                popup.add(defaultItem);
                /// ... add other items
                // construct a TrayIcon
                trayIcon = new TrayIcon(image, "Heinzelnisse", popup);
                trayIcon.setImageAutoSize(true);
                // set the TrayIcon properties
                trayIcon.addActionListener(listener);
                // ...
                // add the tray image
                try {
                    tray.add(trayIcon);
                    setDefaultCloseOperation(javax.swing.WindowConstants.HIDE_ON_CLOSE);
                } catch (AWTException ex) {
                    // usually under webstart
                    Logger.getLogger(HeinzelJFrame.class.getName()).warning("SystemTray: "+ ex.toString());
                }
            }

        } else {
            setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
            if (SystemTray.isSupported() && trayIcon != null) {
                // get the SystemTray instance
                SystemTray tray = SystemTray.getSystemTray();
                tray.remove(trayIcon);
                trayIcon = null;
            }
        }
    }//GEN-LAST:event_jCheckBoxMenuItem1ItemStateChanged

    private void cutNPasteCheckBoxMenuItem2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cutNPasteCheckBoxMenuItem2ActionPerformed
        ControllerBean.getInstance().setCutnpaste(cutNPasteCheckBoxMenuItem2.isSelected());
    }//GEN-LAST:event_cutNPasteCheckBoxMenuItem2ActionPerformed

    private ClipboardService getClipboardService() {
        if (isWebstart() && clipboardService == null) {
            try {
                clipboardService = (ClipboardService) ServiceManager.lookup("javax.jnlp.ClipboardService");
            } catch (UnavailableServiceException ex) {
                Logger.getLogger(HeinzelJFrame.class.getName()).info("ClipboardSercice: " + ex.toString());
            }
        }
        return clipboardService;
    }

    private boolean isWebstart() {
        if (isWebstartTested == false) {
            try {
                ServiceManager.lookup("javax.jnlp.BasicService");
                isWebstart = true;
            } catch (UnavailableServiceException ex) {
                isWebstart = false;
            }
        }
        return isWebstart;
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                final HeinzelJFrame hjf = new HeinzelJFrame();
                hjf.setVisible(true);
            }
        });
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JCheckBoxMenuItem cutNPasteCheckBoxMenuItem2;
    private info.heinzelnisse.he.HeinzelJPanel heinzelJPanel1;
    private javax.swing.JCheckBoxMenuItem jCheckBoxMenuItem1;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenu jMenu2;
    private javax.swing.JMenu jMenu3;
    private javax.swing.JMenu jMenu4;
    private javax.swing.JMenu jMenu5;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JMenuItem jMenuItem2;
    private javax.swing.JMenuItem jMenuItem3;
    private javax.swing.JMenuItem jMenuItem4;
    // End of variables declaration//GEN-END:variables
    private javax.swing.JRadioButtonMenuItem[] jRadioButtonMenuItems;
    private java.awt.TrayIcon trayIcon = null;
    private javax.jnlp.ClipboardService clipboardService = null;
    private boolean isWebstart = true;
    private boolean isWebstartTested = false;
}
