package info.heinzelnisse.he;

import java.util.ArrayList;
import java.util.Collection;

/**
 * Title:        Heinzelnisse.Translation
 * Description:  Desktop Edition of Heinzelnisse
 * Copyright:    Copyright (c) 2002
 * Copyright:    Copyright (c) 2003
 * Company:
 * @author Heiko Klein
 * @version 1.6
 */

public class Translation implements Comparable<Translation> {
    public static final String SEPARATOR = "\t";
    public static final String NO = "NO";
    public static final String DE = "DE";

    private String word = "";
    private String transWord = "";
    private String transArticle = "";
    private String article = "";
    private String options = "";
    private String transOptions = "";
    private String transOther = "";
    private String other = "";
    private String category = "";
    private String lang = DE;

    private boolean isExact = true;

    public Translation() {        
    }
    
    /**
     * @param str TranslationString as 8 SEPARATOR separated Strings containing
     *            wordNO articleNO optionsNO otherNO wordDE articleDE optionsDE otherDE category rank
     */
    public static Translation getTranslationOf(String string, String lang) {
        String[] ss = toStringArray(string);
        int add = 4;
        if (lang == NO) {
            add = 0;
        }
        Translation trans = new Translation();
        trans.lang = lang;
        trans.word = ss[(0+add)%8];
        trans.article = ss[(1+add)%8];
        trans.options = ss[(2+add)%8];
        trans.other = ss[(3+add)%8];
        trans.transWord = ss[(4+add)%8];
        trans.transArticle = ss[(5+add)%8];
        trans.transOptions = ss[(6+add)%8];
        trans.transOther = ss[(7+add)%8];
        trans.category = ss[8];
        return trans;
        
    }
    
    /**
     *
     * This is the list implementation of @see getTranslationOf
     *
     * @param aList list of strings with 8 SEPARATOR separated Strings
     * @return null if aList is null or aList contains no elements
     */
    public static Collection<Translation> stringCollectionToTranslationCollection(Collection<String> aList, String lang) {
        if ((aList == null) || (aList.size() == 0)) {
            return null;
        }
        Collection<Translation> l = new ArrayList<Translation>();
        for (String s : aList) {
            l.add(Translation.getTranslationOf(s, lang));
        }
        return l;
    }

    private static final String[] toStringArray (String string) {
	int maxFields = 9;
	String [] str = new String [maxFields];
	int strPos = 0;
	char sep = SEPARATOR.charAt(0);
	StringBuffer sb = new StringBuffer(32);
	for (int i = 0; i < string.length(); i++) {
	    char chr = string.charAt(i);
	    if (chr == sep) {
		if (strPos < maxFields) {
		    str[strPos++] = sb.toString();
		    sb.setLength(0); // deleting string
		}
	    } else {
		sb.append(chr);
	    }
  	}
	if (strPos < maxFields) {
	    str[strPos] = sb.toString();
	}
	
	return str;
    }

    public String getCategory() {
        return category;
    }

    public String getWord() {
        return word;
    }

    public String getTransWord() {
        return transWord;
    }

    public String getTransArticle() {
        return transArticle;
    }

    public String getArticle() {
        return article;
    }

    public String getTransOptions() {
        return transOptions;
    }

    public String getOptions() {
        return options;
    }

    public String getTransOther() {
        return transOther;
    }

    public String getOther() {
        return other;
    }

    public String getLang() {
        return lang;
    }
    
    public String getNonUniqueIdentifier() {
        return word + "\u0000" + article + "\u0000" + other;
    }
    @Override
    public String toString() {
        return word + "\t" + article + "\t" + other + "\t" + transWord + "\t" + transArticle + "\t" + transOther + "\t" + category + "\t" + lang;
    }
    
    @Override
    public int compareTo(Translation t) {
        return HeinzelTable.getDeNoCollator().compare(getNonUniqueIdentifier(), t.getNonUniqueIdentifier());
    }
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o instanceof Translation) {
            Translation t = (Translation)o;
            return toString().equals(t.toString());
        }
        return false;
    }
    @Override
    public int hashCode() {
        return toString().hashCode();
    }

    /**
     * @return the isExact
     */
    public boolean isIsExact() {
        return isExact;
    }

    /**
     * @param isExact the isExact to set
     */
    public void setIsExact(boolean isExact) {
        this.isExact = isExact;
    }
}
